/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: iofun.h                                                     */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - simple I/O functions class                             */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

/** \file
    This file contains some smart I/O functions
*/

#ifndef _IOFUN_H_
#define _IOFUN_H_

#include <ctime>
#include <cstdio>
#include <cctype>
#include <cstring>
#include <sys/stat.h>

#ifdef _WIN32
#include <windows.h>
#endif

#include "vector3d.h"

/// Simple I/O functions class

class IOFUN
{
    public:

/// conversion of directory separators - MS Windows to Unix 
	static void Win2Unix(char *s);
/// conversion of directory separators - Unix to MS Windows 
	static void Unix2Win(char *s);
    
/// Function to read of a comment till the end of line. It returns 0 or EOF if it is performed.
	static int ReadComm( FILE *stream );
/// Function to read a comment line.
	static void ReadDummy( FILE *stream );
/// Function to read one variable. The type of variable depends on Format, compatible with stdio library.
	static int ReadPar( FILE *stream, const char *Format, void *Par );
/// Function to read two variables. The type of variables depends on Format, compatible with stdio library.
	static int ReadPar( FILE *stream, const char *Format, void *Par1, void *Par2);
/// Function to read three variables. The type of variables depends on Format, compatible with stdio library.
	static int ReadPar( FILE *stream, const char *Format, void *Par1, void *Par2, void *Par3);
/// Function to read four variables. The type of variables depends on Format, compatible with stdio library.
	static int ReadPar( FILE *stream, const char *Format, void *Par1, void *Par2, void *Par3, void *Par4 );
	
/// Function to read double vector "Par" of "3" dimension from FILE "stream". The read values are scaled by multiplier "dScal"
    static int ReadVect3( FILE *stream, double *Par, double dScal = 1. );
/// Function to read float vector "Par" of "3" dimension from FILE "stream". The read values are scaled by multiplier "fScal"
    static int ReadVect3( FILE *stream, float *Par, float fScal = 1. );
/// Function to read long vector "Par" of "3" dimension from FILE "stream". The read values are scaled by multiplier "lScal"
    static int ReadVect3( FILE *stream, long *Par, long lScal = 1 );
/// Function to read int vector "Par" of "3" dimension from FILE "stream". The read values are scaled by multiplier "iScal"
    static int ReadVect3( FILE *stream, int *Par, int iScal = 1 );

/// Function to read double vector "Par" of "n" dimension from FILE "stream". The read values are scaled by multiplier "dScal"
	static int ReadVect( FILE *stream, double *Par, int n, double dScal = 1. );
/// Function to read float vector "Par" of "n" dimension from FILE "stream". The read values are scaled by multiplier "fScal"
	static int ReadVect( FILE *stream, float *Par, int n, float fScal = 1. );
/// Function to read long vector "Par" of "n" dimension from FILE "stream". The read values are scaled by multiplier "lScal"
	static int ReadVect( FILE *stream, long *Par, int n, long lScal = 1 );
/// Function to read int vector "Par" of "n" dimension from FILE "stream". The read values are scaled by multiplier "iScal"
	static int ReadVect( FILE *stream, int *Par, int n, int iScal = 1 );

/// Function to read the new line from FILE "stream" and to store it in table of char "Par". The starting and ending blanc charaters are clipped.
	static int ReadStr( FILE *stream, char *Par );
/// Function to read the string (Par) from FILE "stream". It reads from current pointer of FILE to the end of line or EOF. Number of read characters is returned in "len" 
	static void ReadStrL( FILE *stream, char *Par, int *len );
/// Clipping of the "string" up to the first blanc character or comment "#" character.
	static void ClipString( char *string );
/// Clipping of the ending blanc characters of "string".
	static void ClipFileName( char* string );
/// Compares two strings (str1 and str2). Returns true if they match, false otherwise.
	static bool CompareStrings( char* str1, char* str2 );

/// Returns true (1) if file "name" does exist, 0 otherwise
	static int file_exist( const char *name );
/// Returns pointer to filename without path
	static const char* filename_name( const char *name );
/// Returns pointer to filename extension
	static const char* filename_ext( const char *buf );
/// set/change filename extension
	static char* filename_setext(char *buf, int buflen, const char *ext);
/// returns the length of the path in (path)name
	static int filename_dir( char *name );
/// Returns pointer to the string that contains the date of last file modification/creation
	static const char* filename_date( const char *name );
/// Cross-platform function to fopen function that supports UTF-8 encoded name
	static FILE* fopen( const char*filename, const char*mode );
/// Cross-platform function to stat() a file using a UTF-8 encoded name or value
	static int stat(const char* f, struct stat *b);
	
/// The implementation of nonstandard strlcpy function
	static size_t strlcpy(char *d, const char *s, size_t bufsize);
	
/// Returns number of lines of text file from current pointer to EOF
	static int nLines( FILE * stream );
/// Returns number of columns of text file (number of strings in the first line). Names of columns are stored in Par array.
	static int nColumns( FILE * stream, char Par[][32] );
/// Checks if string (cc) is blank (1/0)
	static int isstrblank( char *cc, int iLen );
	
/// Function to read VECTOR_3D Vec components from FILE "stream". The read values are scaled by multiplier "dScal"
	static void ReadVect3( FILE *stream, VECTOR_3D *Vec, double dScal = 1. );
	
/////////////////////////////////////////////////////////////////////////////
//
//                   Pascal style "readln" and "read" functions
//                   by P.Blaszczyk - modif. by TGG (2002)
//
/////////////////////////////////////////////////////////////////////////////

// Pascal style "readln" functions

/// Pascal style "readln" function to read string
	static int readln( FILE *ff, char *value_x );
/// Pascal style "readln" function to read int
	static int readln( FILE *ff, int *value_x );
/// Pascal style "readln" function to read long
	static int readln( FILE *ff, long *value_x );
/// Pascal style "readln" function to read float
	static int readln( FILE *ff, float *value_x );
/// Pascal style "readln" function to read double
	static int readln( FILE *ff, double *value_x );
/// Pascal style "readln" function to read long double
	static int readln( FILE *ff, long double *value_x );
/// Pascal style "readln" function to read comment
	static int readln( FILE *ff );
/// Pascal style "readln" function to read VECTOR_3D components
	static int readln( FILE *ff, VECTOR_3D *vec );

// Pascal style "read" functions

/// Pascal style "read" function to read string
	static int read( FILE *ff, char *value_x );
/// Pascal style "read" function to read int
	static int read( FILE *ff, int *value_x );
/// Pascal style "read" function to read long
	static int read( FILE *ff, long *value_x );
/// Pascal style "read" function to read float
	static int read( FILE *ff, float *value_x );
/// Pascal style "read" function to read double
	static int read( FILE *ff, double *value_x );
/// Pascal style "read" function to read long double
	static int read( FILE *ff, long double *value_x );
/// Pascal style "read" function to read VECTOR_3D components
	static int read( FILE *ff, VECTOR_3D *vec );

};

#endif /*_IOFUN_H_*/
