/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2015                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2015                           */
/*********************************************************************/
/*                                                                   */
/* File: ms2data.h                                                   */
/*                                                                   */
/* Author: D.Mieszalski, modif: T.Grabowski                          */
/*                                                                   */
/* Contents - ms2 data I/O class                                     */
/*                                                                   */
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

/** \file
    This file contains ms2 (geometry info) data structure manipulation.
*/

#ifndef _MS2_DATA_H_
#define _MS2_DATA_H_

#include <map>
#include <iostream>
#include <fstream>

#include "iofun.h"
#include "conect_data.h"

/// MS2 data structure manipulation

class MS2_Data : public MS2_Data_Struct, public IOFUN
{
    protected:
        static const int CURRENT_VERSION = 53;                 ///< current version of ms2data module
        int iVersion;                                          ///< version of read file
        char cComment[256];                                    ///< message text in case of error
        std::string RemoveExt( std::string filename );         ///< removes the filename extention

        static std::map<int,const char*> AirfoilTypeStr;       ///< file, NACA, table
        //fuselage string
        static std::map<int,const char*> FusTypeStr;           ///< file, parameters
        static std::map<int,const char*> FusCvrStr;            ///< spine, up cvr etc.
        static std::map<int,const char*> FusCvrTypeStr;        ///< ellipse, airfoil etc.
        static std::map<int,const char*> FusAirfoilSide;       ///< up , down
        //wing strings
        static std::map<int,const char*> WingTypeStr;          ///< sections, modules, nacelle
        static std::map<int,const char*> WingModTypeStr;       ///< root, equi line, arc, line
        static std::map<int,const char*> WingModPanConcStr;    ///< paneling distribution in module wings modules ( left, right center )
        static std::map<int,const char*> WingDivSourceStr;     ///< master, slave
        static std::map<int,const char*> WingSurfSideStr;      ///< all, top, bottom
        static std::map<int,const char*> WingRibTypeStr;       ///< open, external, both
        static std::map<int,const char*> WingSectionDivStr;    ///< linear, cosine etc.
        static std::map<int,const char*> NacelleSwirlStr;      ///< swirl up, swirl down
        static std::map<int,const char*> NacelleRangeStr;      ///< swirl range: full, half

        int GetIndpWingNo( void );                             ///< returns number of indepedentend wings (MASTER & INDEPENDENT)
        int GetDpWingNo( void );                               ///< returns number of depedentend wings (SLAVE)

    public :
        bool WriteGeneral( std::ostream &f );                       ///< writes general data, incuding reference values

        bool WriteWing( std::ostream &fout, MS2_Wing wing, int no );       ///< writes wing's data
        bool WriteWingModule( std::ostream &fout, MS2_ModWingModule mod ); ///< writes modular wing's data 
        bool WriteSection( std::ostream &fout, MS2_Section sect );         ///< writes data of wing built from sections
        bool WriteProfile( std::ostream &fout, MS2_Profile airfoil );      ///< writes airfoil data
        bool WriteFuselage( std::ostream &fout );                          ///< writes fuselage data
        bool WriteConns( std::ostream &fout, bool bCom = true );           ///< writes connections data
        bool WriteWakes( std::ostream &fout, bool bCom = true );           ///< writes extra wake infos
        bool WriteEmbededProfiles( std::ostream &fout, std::vector<MS2_Profile> *airfoil ); ///< writes embeded airfoil geometry

        virtual bool ReadWing( FILE *t, MS2_Wing &wing, int iW, bool bRestrictive, bool auto_mod );     ///< reads wing data (see \ref MS2_Wing)
        bool ReadWingModule( FILE* t, MS2_ModWingModule &mod, bool bRestrictive );                      ///< reads Wing Module data (see \ref MS2_ModWingModule)
        bool ReadSection( FILE *t, MS2_Section &sect );                                                 ///< reads Wing Section data (see \ref MS2_Section)
        bool ReadProfile( FILE *t, MS2_Profile &airfoil, const char* desc, bool bRestrictive );         ///< reads Profile data structure (see \ref MS2_Profile)
        bool ReadProfile( FILE *t, MS2_Profile &airfoil );                                              ///< reads Profile data in case \ref AIRFOIL_FILE type
		
        bool ReadFusFromParams( FILE *t, bool bRestrictive);                                            ///< reads Fuselage data in case fuselage defined using modules
        bool ReadFusEllipse( FILE *t, MS2_SuperEllipse &SupEll, const char* desc);                      ///< reads fuselage elipse data
        bool ReadFusAirfoil( FILE *t, MS2_FusAirfoil &airfoil, const char* desc, bool bRestrictive );   ///< reads airfoil data used to define fuselage shape
        bool ReadChordDivision( FILE* t, MS2_Wing &wing, bool bRestrictive );                           ///< reads user define chord division (see \ref DT_USER)
       // bool ReadNacelle( FILE *t, MS2_Nacelle & nac, int no );

        static void ClearFusCurve( MS2_FusCurve &cvr, int elltype );       ///< cleans \ref MS2_FusCurve object (fuselge module)
        static void SetFusCurveDefs( MS2_FusCurve &cvr, int elltype );     ///< sets \ref MS2_Profile object (airfoil)
        static void ClearGeneral( MS2_General &gen );                      ///< cleans \ref MS2_General object (general data)

        virtual void Comment( void );                                       ///< displays comment (warning or information)

        MS2_Data( void );
        virtual ~MS2_Data( void );

        void Clear(void);                                                   ///< clear local variables
        void SetDefaults( void );                                           ///< set default reference values
        bool Read( const char * file, bool bRestrictive, bool auto_mod  );  ///< reads ms2 file
        bool Write( const char *file );                                     ///< writes ms2 file

	int GetVersion( void ){ return iVersion; }                          ///< returns ms2 file version
        int GetNWings( void ){ return Wings.size(); }                       ///< returns number of wings
        int GetNConns( void ){ return Conns.size(); }                       ///< returns number of connections
        int GetNWakes( void ){ return Wakes.size(); }                       ///< returns number of wake infos
        int GetNProfs( void ){ return Profiles.size(); }                    ///< returns number of embeded airfoils
        std::string WingSurfName( int n );                                  ///< returns wing surface type (name) - see \ref MS2WingSurf
        std::string DivTypeName( int div );                                 ///< returns chordwise division type (name) - see \ref MS2DivType

        static void SetPrfDefs( MS2_Profile &prf );                         ///< sets \ref MS2_Profile object (airfoil)
        static void SetWingDefs( MS2_Wing &wing );                          ///< sets \ref MS2_Wing object (wing definition)
        static void SetSectionDefs( MS2_Section &sect );                    ///< sets \ref MS2_Section object (wing section definition)
        static void SetModuleDefs( MS2_ModWingModule &mod );                ///< sets \ref MS2_ModWingModule object (wing module definition)
        static void SetNacelleWingDefs( MS2_NacelleWing &nac );             ///< sets \ref MS2_NacelleWing object (jet nacelle definition)
        static void SetFuselageDefs( MS2_Fuselage &fus );                   ///< sets \ref MS2_Fuselage object (fuselage definition)
        static void SetConnectionDefs( MS2_Conn &conn );                    ///< sets \ref MS2_Conn object (wings connection)
        static void SetWakeDefs( MS2_Wake &wake );                          ///< sets \ref MS2_Wake object (wake information)

        static void ClearPrf( MS2_Profile &prf );                           ///< cleans \ref MS2_Profile object (airfoil)
        static void ClearWing( MS2_Wing &wing );                            ///< cleans \ref MS2_Wing object (wing definition)
        static void ClearSection( MS2_Section &sect );                      ///< cleans \ref MS2_Section object (wing section definition)
        static void ClearModule( MS2_ModWingModule &mod );                  ///< cleans \ref MS2_ModWingModule object (wing module definition)
        static void ClearNacelleWing( MS2_NacelleWing &nac );               ///< cleans \ref MS2_NacelleWing object (jet nacelle definition)
        static void ClearFuselage( MS2_Fuselage &fus );                     ///< cleans \ref MS2_Fuselage object (fuselage definition)
        static void ClearConnection( MS2_Conn &conn );                      ///< cleans \ref MS2_Conn object (wings connection)
        static void ClearWake( MS2_Wake &wake );                            ///< cleans \ref MS2_Wake object (wake information)

        static void CONNECT2MS2_Conn( CONNECT_DATA &Con, MS2_Conn &newConn ); ///< converts CONNECT_DATA to MS2_Conn
        static void MS2_Conn2CONNECT(  MS2_Conn &Conn, CONNECT_DATA &newCon );///< converts MS2_Conn to CONNECT_DATA

        static int find_section( FILE *ff, const char* sec );   ///< to find the section in ms2 file - imported from "mesh2.h"
        static bool has_section( FILE *t, const char * sec );   ///< check if section exist

        static std::string ConnTypeName( int n );                ///< returns connection type name
        static std::string WakeTypeName( int n );                ///< returns wake info type name
        static std::string TruncateQuotMarks( std::string str ); ///< truncates the quote marks from string
        static std::string RemoveWhiteSpaces( std::string str ); ///< removes white chars from string
        static std::string GetNACANoAsString( int no );          ///< converts NACA profile number to string
};

#endif


