/*********************************************************************/ 
/*                                                                   */ 
/*             Config files library - (C) TGG 2015                   */ 
/*                                                                   */ 
/*********************************************************************/ 
/*                          Warszawa, 2015                           */ 
/*********************************************************************/ 
/*                                                                   */
/* File: ms2data_struct.h                                            */
/*                                                                   */
/* Author: D.Mieszalski, modif: T.Grabowski                          */
/*                                                                   */
/* Contents - ms2 (geometry info) data structure                     */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

/** \file
    This file contains ms2 (geometry info) data structure.
*/

#ifndef _MS2_DATA_STRUCT_H_
#define _MS2_DATA_STRUCT_H_

#include <string>
#include <vector>

enum MS2SymmOpt     /// symmetry flag
{
    SYM_OPT_SYM = 0,   ///< symmetrical object
    SYM_OPT_RIGHT = 1, ///< right half
    SYM_OPT_LEFT = -1  ///< left half
};

enum MS2OutOpt      /// output type flag
{
    OUT_OPT_NODES = 0,  ///< mesh written as nodes
    OUT_OPT_PANELS = 1  ///< mesh written as panels
};

enum MS2WingSurf    /// section wing surface side
{
    WS_ALL = 0,     ///< both surfaces (up and bottom)
    WS_TOP,         ///< upper surface (up)
    WS_BOTTOM       ///< lower surface (bottom)
};

enum MS2DivType     /// section wing type chord division
{ 
    DT_LINEAR = 0,   ///< linear (constant chordwise length)
    DT_COSINE,       ///< constant chordwise angle projection
    DT_USER,         ///< user defined
    DT_PROFILE,      ///< based on airfoil data
    DT_AUTO          ///< "unknown" in MS2Edit, in SLAVE wing case (see \ref MS2WingDivSource)
};

enum MS2SpanDivType /// section wing type chord division
{
    SDT_LINEAR = 0, ///< linear (constant spanwise length)
    SDT_COSINE,     ///< constant spanwise angle projection (0,pi)
    SDT_COSINE_EXT, ///< constant spanwise angle projection (pi/2, pi)
    SDT_COSINE_INT  ///< constant spanwise angle projection (0, pi/2)
};

enum MS2ConnType    /// connection type
{
    CON_H = 0,    ///< horizontal (e.g. T tail)
    CON_V,        ///< vertical (e.g. winglets)
    CON_P         ///< parallel
};

enum MS2WakeType    /// wake type
{
    WAKE_H = 0,   ///< horizontal (e.g. wing, horzontal tail)
    WAKE_V        ///< vertical (e.g. vertical stabilizer)
};

enum MS2AirfoilSide   /// airfoil side for modules definition (see \ref MS2_FusAirfoil::Side)
{
    AS_UP = 0,    ///< upper part
    AS_DOWN       ///< lower part
};

enum MS2FusCvrType   /// fuselage build from parameters curve's type
{
    ELLIPSE = 0,
    AIRFOIL,
    CONSTANT
};

enum MS2AirfoilSource /// airfoil source
{
    AIRFOIL_FILE = 0,    ///< airfoil geometry defined in the separate file
    AIRFOIL_NACA,        ///< airfoil geometry computed for 4- and 5-digit NACA profiles
    AIRFOIL_DATA         ///< airfoil geometry embeded in MS2 file
};

enum MS2FusCvr       /// type of fuselage outline curve
{
    CRV_SPINE = 0,
    CRV_UP_CONT,
    CRV_SIDE_CONT,
    CRV_DOWN_CONT,
    CRV_CV_FACT_UP,
    CRV_CV_FACT_DOWN
};

enum MS2FusEllipseType  /// Fuselage curve (super ellipse ) type
{
    ELLIPSE_NORMAL = 0,
    ELLIPSE_SPINE,
};

enum MS2FusType         /// fuselage type
{
    FT_FILE = 0,        ///< from *.f file
    FT_PARAMS           ///< built from parameters
};

enum MS2FusSecDivType   /// type of fuselage section division
{ 
    FDT_DATA = 0,       ///< coordinates defined by user (\ref FT_FILE) or by parameters (\ref FT_PARAMS)
    FDT_ARC,            ///< coordinates calculated using constant arc length rule
    FDT_ANGLE           ///< coordinates calculated using constant angle rule
};

enum MS2WingType       /// type of wing
{
    WING_TYPE_SECTIONS = 0,    ///< wing defined by sections
    WING_TYPE_MODULES,         ///< wing defined by modules
    WING_TYPE_NACELLE          ///< jet engine nacelle
};

enum MS2WingDivSource  /// wing chordwise division source used in wing in case of intersection with fuselage (see \ref MS2_Wing::DivSource)
{
    WING_DIV_MASTER = 1,        ///< wing enforces fuselage's chord division
    WING_DIV_SLAVE = 0,         ///< fuselage enforces wing's chord division
    WING_DIV_INDEPENDENT = -1   ///< wing has its own division and doesn't enforce division to fuselage
};

enum MS2WingRibType    /// closing ribs type
{
    WING_RIB_OPEN = 0,   ///< no closing ribs (e.g. wing is open and connected with winglets)
    WING_RIB_EXTERNAL,   ///< only external rib  (typical situataion external rib on wing tip)
    WING_RIB_BOTH,       ///< both ribs are present (e.g. in case of slotted flap)
    WING_RIB_INTERNAL    ///< only internal rib
};

enum MS2WingModuleType  /// types of modules used in module wing creation
{
    WING_MOD_TYPE_ROOT = 0,
    WING_MOD_TYPE_EQUI_LINE,
    WING_MOD_TYPE_LINE,
    WING_MOD_TYPE_ARC
};

enum MS2WingModulePanConc   /// module wing's module panels distribution
{
    WING_MOD_PAN_CONC_LEFT  = 0,
    WING_MOD_PAN_CONC_CENTER,
    WING_MOD_PAN_CONC_RIGHT
};


enum MS2SwirlType   /// nacelle swirl direction
{
    SWIRL_UP = 0,
    SWIRL_DOWN
};

enum MS2SwirlRange  ///nacelle swirl range
{
    FULL_NACELLE = 0,
    HALF_NACELLE
};

struct MS2_Ref                  /// Reference values
{
    bool AutoComp;              ///< compute reference parameters automatically (flag)
    double S;                   ///< reference (main wing) area
    double B;                   ///< wing span
    double MAC;                 ///< Mean Aerodynamic Chord
    double MX;                  ///< X coordinate of reference point for moment calculation
    double MZ;                  ///< Z coordinate of reference point for moment calculation
    double Scale;               ///< scale of geometry (default Scale=1)
};

struct MS2_Profile              /// Profile definition
{
    int iType;                  ///< type of profile data: 0 - from file, 1 - NACA 4 or 5 digits airfoil, 2 - data embedded in MS2 file
    std::string Name;           ///< full name with file path
    std::string ShortName;      ///< airfoil name without file path
    int NACAnumber;             ///< code number of NACA - 4 or 5 digits in case iType == AIRFOIL_NACA
//    int No;                     ///< embedded profile number ( 0 - n )
    std::vector<double> Xd;     ///< X coordinate of lower contour of airfoil
    std::vector<double> Xg;     ///< X coordinate of upper contour of airfoil
    std::vector<double> Zd;     ///< Z coordinate of lower contour of airfoil
    std::vector<double> Zg;     ///< Z coordinate of upper contour of airfoil
};

struct MS2_Section              /// definition of section (used in "Section Wing")
{
    MS2_Profile Profil;         ///< airfoil data
    double r;                   ///< distance through wing curvature from root section (currently only for modular wings)
    double X;                   ///< X coordinate of origin [m]
    double Y;                   ///< Y coordinate of origin [m]
    double Z;                   ///< Z coordinate of origin [m]
    double RotX;                ///< rotation angle along X axis [deg]
    double RotY;                ///< rotation angle along Y axis [deg]
    double RotZ;                ///< rotation angle along Z axis [deg]
    double Chord;               ///< chord [m]
    int No;                     ///< section number
};

struct MS2_SecWing              /// wing build from sections ( WING_TYPE_SECTIONS )
{
    std::vector<MS2_Section> Sections; ///< wing sections set
    int RibNr;                         ///< not defined in ms2 file - inner variable
    int DivType;                       ///< type of spanwise division - see MS2SpanDivType
};

struct MS2_ModWingModule    /// module used for "module wing" definition
{
    int Type;               ///< type of module - see MS2WingModuleType
    double X0;              ///< x coordinate of origin
    double Y0;              ///< y coordinate of origin
    double Z0;              ///< z coordinate of origin
    double XRef;            ///< first definition curve's start point location [%MAC]
    double C;               ///< Mean Aerodynamic Chord (MAC)
    double Alfa;            ///< twist angle [deg]
    double Fi;              ///< skew angle [deg]
    double Gamma;           ///< dihedral angle [deg]
    double L;               ///< length
    double R;               ///< radius
    int N;                  ///< number of division
    double NDist;           ///< division dist
    MS2_Profile Profil;     ///< airfoil data
    int SpanPanConc;        ///< paneling distribution concentration( Left, center, right )
    int UseMunk;            ///< (0,1) Use Munk minimum induced drag theorem
};

struct MS2_ModWing          /// wing defined by modules
{
    std::vector<MS2_ModWingModule> Modules;   ///< vector (list) of modules used to define wing
};

struct MS2_NacelleWing              /// data of nacelle type of wing
{
    MS2_Profile Profil;             ///< airfoil data
    int NUp;                        ///< division of top part - number of wing sections
    int NDown;                      ///< division of bottom part - number of wing sections
    int SwirlType;                  ///< swirl direction (external or internal)
    int SwirlRange;                 ///< full or half
    double X0;                      ///< x coordinate of origin
    double Y0;                      ///< y coordinate of origin
    double Z0;                      ///< z coordinate of origin
    double L;                       ///< length (chord)
    double CvFactUp;                ///< convexity factor of top part
    double CvFactDown;              ///< convexity factor of bottom part
    double Y;                       ///< Y dimension (semiaxis longwise Y)
    double ZUp;                     ///< Z up dimension (upper semiaxis longwise Z)
    double ZDown;                   ///< Z down dimension (lower semiaxis longwise Z)
    double fi;                      ///< roll rotation angle [deg]
    double theta;                   ///< pitch rotation angle [deg]
    double psi;                     ///< yaw rotation angles [deg]
};

struct MS2_Wing             /// wing definition
{
    int Type;               ///< type of the wing ( see MS2WingType )
    int DivSource;          ///< 1 - master, 0 - slave, -1 - independent (\see MS2WingDivSource ) used in fuselage-wing intersection
    int Active;             ///< 1 - active, 0 - not active
    std::string Name;       ///< name of wing
    int ID;                 ///< unique object ID

    int ComputeInt;	        ///< 1 - compute fuselage-wing intersection, 0 - wing is glued to the fuselage
    int LongeronNo;         ///< fuselage longeron No which will be connected with wing's leading edge
    int SmartBypass;        ///< smart deflection of longerons to bypass the wing  (0 - no, 1 - yes, modifies the section points)
    int BypassDepth;        ///< max. depth of bypassing - number of longerons to be deflected
    int SurfType;           ///< wing surface to create ( see MS2WingSurf )
    int RibType;            ///< see MS2WingRibType

    int DivNo;                      ///< number of chord division points
    int DivType;                    ///< chord division type \see MS2DivType
    std::vector<double> UserDiv;    ///< user defined chord division

    double TwistRef;        ///< twist reference point (0...1)

    int UseMunk;            ///< (0,1) Use Munk minimum induced drag theorem
    double SRef;            ///< reference area for min induced drag calculation
    double CzDes;           ///< design lift force coefficient
    MS2_SecWing Sec;        ///< Section wing definition
    MS2_ModWing Mod;        ///< Module wing definition
    MS2_NacelleWing Nac;    ///< Nacelle definition
};

struct MS2_General          /// general data for ms2 structure (geometry)
{
    MS2_Ref RefVal;          ///< reference values
    int SymmOpt;             ///< symmetry ( see MS2SymmOpt )
    int OutOpt;              ///< output type ( see MS2OutOpt )
    std::string InpFileName; ///< output [*.inp] file name
};

struct MS2_SuperEllipse         /// data for module of super ellipse type
{
    int EllType;                ///< Ellipse type - ( spine ellipse or normal ellipse )
    int NF;                     ///< division number front
    int NR;                     ///< division number rear
    double L;                   ///< front length
    double H;                   ///< front height
    double CvFactF;             ///< convexity factor front
    double CvFactR;             ///< convexity factor rear
};

struct MS2_FusAirfoil : public MS2_Profile  /// data of airfoil taken to fuselage geometry generating
{
    int Side;                   ///< airfoil's side
};

struct MS2_FusCurve             /// fuselage outline curve
{
    int Type;                   ///< curve type ( see MS2FusCvrType )
    MS2_SuperEllipse Ellipse;   ///< data used when Type is  ELLIPSE
    MS2_FusAirfoil Airfoil;     ///< data used when Type is  AIRFOIL
    double Factor;              ///< data used when Type is  CONSTANT
};

struct MS2_Fuselage_Params      /// fuselage parameters
{
    //main parameters
    std::string Name;               ///< fuselage name
    double Scale;                   ///< general scale
    double ScaleX;                  ///< scale longwise X axis
    double ScaleY;                  ///< scale longwise Y axis
    double ScaleZup;                ///< scale longwise Z axis Up
    double ScaleZdown;              ///< scale longwise Z axis Down
    double X0;                      ///< x coordinate of nose origin
    double Y0;                      ///< y coordinate of nose origin
    double Z0;                      ///< z coordinate of nose origin
    double BendLength;              ///< bend distance
    double BendAngle;               ///< bend angle [deg]
    int Nup;                        ///< division of top part
    int Ndown;                      ///< division of bottom part
    double Length;                  ///< fuselage length
    //curves
    MS2_FusCurve SpineCvr;          ///< X contour line
    MS2_FusCurve SideCvr;           ///< Y contour line
    MS2_FusCurve UpCvr;             ///< Z Upper contour line
    MS2_FusCurve DownCvr;           ///< Z Lower contour line
    MS2_FusCurve CvFactUp;          ///< ConvexityFactor curve (upper)
    MS2_FusCurve CvFactDown;        ///< ConvexityFactor curve (lower)
};

struct MS2_Fuselage_File            /// fuselage data file
{
    std::string Name;               ///< fuselage data (geometry) filename
};

struct MS2_Fuselage                 /// fuselage definition
{
    bool Exists;                    ///< fuselage presence flag
    int Active;                     ///< 1 - active, 0 - not active
    int Type;                       ///< MS2FusType
    std::vector<double> UserDiv;    ///< user defined divisions
    MS2_Fuselage_File File;         ///< fuselage from *.f file
    MS2_Fuselage_Params Params;     ///< fuselage build from parameters

    int iDivSectionType;            ///< type of section division - see MS2FusSecDivType
    int iSecN;                      ///< number of points that define the section
};

struct MS2_Conn     /// connection
{
    std::string Name;   ///< name of particular connection
    int Active;         ///< 1 - active, 0 - not active
    int Type;           ///< Type of connection (horizontal, vertical, parallel) - according to MS2ConnType
    int Wing1No;        ///< number of 1'st wing
    int Wing2No;        ///< number of 2'nd wing
    int Wing1Rib;       ///< number of wing's 1 rib connected to wing 2
    int Wing2Rib;       ///< number of wing's 2 rib connected to wing 1
    int Wing2LongLE;    ///< number of wing's 2 longeron connected to leading point of wing 1
    int Wing2LongTE;    ///< number of wing's 2 longeron connected to trailing point of wing 1
    int Wing2Surf;      ///< wing 2 surface for modification (0-bottom, 1-top in horizontal connection or 0-exterior, 1-interior in vertical connection )
};

struct MS2_Wake     /// wake
{
    std::string Name;   ///< name of particular wake information
    int Active;         ///< 1 - active, 0 - not active
    int Type;           ///< type of wake plane WAKE_H (horizontal) or WAKE_V (vertical)
    int Wing1No;        ///< number of 1'st wing (INDEPENDENT)
    int Wing2No;        ///< number of 2'nd wing
    int Wing1Rib;       ///< number of wing's 1 rib connected to wing 2 by wake line
    int Wing2Rib;       ///< number of wing's 2 rib which modifies wake line
    int Wing2Surf;      ///< wing 2 surface for wake modification (0-bottom, 1-top in case of vertical wake or 0-exterior, 1-interior in case of horizontal wake )
};

struct MS2_Data_Struct                  /// MS2 data structure definition
{
    MS2_General General;                ///< general geometric data
    MS2_Fuselage Fuselage;              ///< fuselage data
    std::vector<MS2_Wing> Wings;        ///< wings data
    std::vector<MS2_Conn> Conns;        ///< connections data
    std::vector<MS2_Wake> Wakes;        ///< special info for wake calculation
    std::vector<MS2_Profile> Profiles;  ///< embedded profiles
};

#endif /* _MS2_DATA_STRUCT_H_*/
