/*********************************************************************/
/*                                                                   */
/*             Config files library - (C) TGG 2018                   */
/*                                                                   */
/*********************************************************************/
/*                          Warszawa, 2018                           */
/*********************************************************************/
/*                                                                   */
/* File: panukl_struct.h                                             */
/*                                                                   */
/* Author: T.Grabowski                                               */
/*                                                                   */
/* Contents - structures definitions                                 */
/*                                                                   */ 
/*                                                                   */
/*********************************************************************/
/*                                                                   */
/*                                                                   */

/** \file
    This file contains class for PANUKL input and output files manipulation
*/

#ifndef _PANUKL_STRUCT_H_
#define _PANUKL_STRUCT_H_

#include <string>
#include <vector>
#include <cstdio>

#include <errno.h>

#include "iofun.h"
#include "memfun.h"
#include "panel.h"

#define MaxWakePanelNumber 1000  ///< limit of numbers of panels in wake strip
#define MaxWakesNumber 1000      ///< limit of numbers of wake strips

/// structure that contains data of main airplane objects (wings, fuselage)

struct MS2_OBJ
{
	std::string Name;          ///< name of the object (e.g. "main wing")
	std::string NameID;        ///< nameID - global object name
	int n1;                    ///< the first panel number in the object
	int n2;                    ///< the last panel number in the object
	int iActive;               ///< a flag - if the object is active mesh is created
	int iClosingRib;           ///< closing rib flag - see \ref MS2WingRibType
	
	int ID;                    ///< global number of the object (e.g. several subobjects with the same ID make up one gobal object)
	int iType;                 ///< typw of the object (0 - wing, 1 - fuselage)
	
	int SurfType;              ///< wing surface to create ( see \ref MS2WingSurf : \ref WS_ALL = 0, \ref WS_TOP = 1, \ref WS_BOTTOM = 2 )
	int RibNr;                 ///< number of ribs
	int DivNo;                 ///< number of chord division points
	int DivType;               ///< chord division type ( see \ref MS2DivType )
};

#define VERSION  4.00          ///< \ref VERSION of the data file [.inp] format currently is equal to 4.00

/// main class for PANUKL input and output files manipulation

class GRID_ARRAYS : public MEMFUN, IOFUN
{
	public:
	
	GRID_ARRAYS( void );       ///< default constructor
	~GRID_ARRAYS( void );      ///< destructor

	float fVer;                ///< version number of data file [.inp] format: from data file on input, defined in \ref VERSION on output
	double dWsp_sk;            ///< scale factor (default: 1)
	
///@{ \name Reference data
	double dSS;                ///< reference area
	double dCa;                ///< mean aerodynamic chord (MAC)
	double dBs;                ///< wingspan
	double dXca25;             ///< x coordinate of reference point for moment calculation (usually 0.25 MAC)
	double dZca25;             ///< z coordinate of reference point for moment calculation (usually 0.25 MAC)
///@}	

///@{ \name Main arrays dimensions
	int	iMaxPanelNumber;       ///< number of body (airplane) panels
	int	iMaxWakesNumber;       ///< maximum of numbers of wake strips
	int	iMaxWakePanelNumber;   ///< maximum number of panels in wake strip
	int	iMaxBodyNeighbours;    ///< number of side body-wake neighbours
///@}
	
///@{ \name Main arrays and objects	
	double	*dX;               ///< main grid (+ wake for Panukl) X[i,j,k], i=panel number, j=corner number, k=coordinate number
	double	*dXw;              ///< wake grid (Xw[i,j,k])
	double	*dXkw;             ///< side body-wake neighbours (panels)

	std::vector<PANEL> Panels;                       ///< panels array
	std::vector<VECTOR_3D> Points;                   ///< nodes array
	std::vector<MS2_OBJ> Objs;                       ///< Objects (\ref MS2_OBJ) vector
///@}	

///@{ \name Wake data
	int iLiczp;           ///< number of wake panels
	int iLiczw;           ///< number of wake strips
	    
	int *iNkw;            ///< numbers of panels in Xkw (wake grid)
	int *iKw;             ///< info about side body wake neighbours iKw[i,j] \n i - side body-wake neighbour number, \n j = 0 - number of panels in this neighbour strip \n j = 1,2 - numbers of the first and the last object panel neighbouring with the wake
	    
	int *iNeigh;	      ///< neighbours numbers array (Nww2,8)
	int *iNnww;           ///< array containing upper surface panels neighbouring with the wake\n
	                      ///< iNnww[i,j] \n
	                      ///< i - strip number \n
	                      ///< j=1 - upper panel number \n
	                      ///< j=2 - edge number of upper panel neighbouring with the wake \n
	                      ///< j=3 - wake panel number that is the neighbour of upper panel (j=1)
	int *iNpas;           ///< number of panels in the wake strip [liczw]
	int *iIpas;           ///< wake panels numbers in the strip [liczw, MaxWakePanelNumber]
///@}

///@{ \name	Working variables and arrays for Panukl calculation
	int 	iN;	        ///< = \ref iMaxPanelNumber;
	int 	iNp;		///< Number of nodes;
	
	int 	iNw;		///< = iN + iLiczw;	   # airplane + wake (up) (1 panel/strip)
	int 	iNw2;		///< = iNw + iLiczw;   # airplane + wake (up+down) (1 panel/strip)
	int 	iNww;		///< = iN + iLiczp;	   # airplane + wake (up)
	int 	iNww2;		///< = iNww + iLiczw;  # airplane + wake (up) + wake (down 1 panel/strip)

	double	*dAi;		///< coefficient array (N x Nww)
	double	*dRhs;		///< right hand side vector input - result on output (Nww2)
	double	*dRh;		///< working area (N)
	int 	*IPIV;		///< pivot table
	int 	iSym;		///< symmetry flag
///@}
	
///@{ \name Collocation points
	double	*dXp;
	double	*dYp;
	double	*dZp;
///@}

///@{ \name Panels centers
	double	*dX0;
	double	*dY0;
	double	*dZ0;
///@}
	
///@{ \name Directional cosines
	double	*dAlf1;
	double	*dAlf2;
	double	*dAlf3;

	double	*dBet1;
	double	*dBet2;
	double	*dBet3;
	
	double	*dGam1;
	double	*dGam2;
	double	*dGam3;
///@}

///@{ \name Velocities, pressure, and other working arrays
	double	*dQx;      ///< tangential undisturbed velocity - x coordinate
	double	*dQy;      ///< tangential undisturbed velocity - y coordinate
	double	*dQz;      ///< tangential undisturbed velocity - z coordinate
	double	*dSig;     ///< source (singularity) value

	double	*dVxd;     ///< "x" component of induced velocity (from doublet distribution)
	double	*dVyd;     ///< "y" component of induced velocity (from doublet distribution)
	double	*dVzd;     ///< "z" component of induced velocity (from doublet distribution)
	
	double	*dVVVV;    ///< undisturbed velocity
	double	*dCpp;     ///< pressure coefficient
	double	*dCxi;     ///< induced drag coeffcient (from strip)
	double	*dCzi;     ///< strip lift coeffcient
	double	*dVv2;     ///< square of total velocity on the panel
	double	*dMach;    ///< lacal panel Mach number
	
	double	*dSss;     ///< panels areas
	double	*dSs_rzut; ///< projection areas of panels on plane xy
	double	*dSs_cpr;  ///< strip area
	double	*dSs_cpr0; ///< strip wetted area
	
	double	*dZzz;
	double	*dObr; 
	double	*dCpb;
	double	*dCpr;     ///< normal force coefficient acting on the strip
	double	*dCmr;     ///< twisting moment coefficient acting on the strip
	double	*dCay;     ///< strip chord
	double	*dCzr;     ///< "z" force coefficient acting on the strip
	double	*dCyr;     ///< "y" force coefficient acting on the strip
	double	*dCxr;     ///< "z" force coefficient acting on the strip
///@}	

///@{ \name Functions to read, write and convert data
	int ReadINP( char *FileName );                    ///< reads [.inp] file from pathname FileName
	int ReadIN1( FILE *File );                        ///< reads [.inp] data from stream FILE - data written as panels
	int ReadIN2( FILE *File );                        ///< reads [.inp] data from stream FILE - data written as nodes
	int ReadDAT( char *FileName, int iProg );         ///< reads [.dat] file from pathname FileName - body and wake panels stored in dX array \n iProg: 1 initialization for Panukl program, 2 initialization for Press program
	int ReadDATw( char *FileName );                   ///< reads [.dat] file from pathname FileName - body and wake panels stored in separated arrays (for visualization)

	int WriteInpFromPanels( char *FileName, int iScale = 0 ); ///< writes [.inp] file to pathname FileName - data stored in panels
	int WriteInpFromPoints( char *FileName, int iScale = 0 ); ///< writes [.inp] file to pathname FileName - data stored in nodes
	int WriteDAT( char *FileName );                   ///< writes [.dat] file to pathname FileName
	
	void Panels2Points( void );                       ///< converts panels (std::vector<\ref PANEL>) to nodes (std::vector<\ref VECTOR_3D>)
	void Points2Panels( void );                       ///< converts nodes (std::vector<\ref VECTOR_3D>) to panels (std::vector<\ref PANEL>) 
	void Corner2Points( int *iD, VECTOR_3D *P );      ///< assigns panel corner P to node number iD

	void Panels2Array( void );                        ///< converts panels (std::vector<\ref PANEL>) to array (double X[i,j,k])
	void Array2Panels( void );                        ///< converts array (double X[i,j,k]) to panels (std::vector<\ref PANEL>)
	
	int ReadObjs( FILE *f );                          ///< Reads objects
	void WriteObjs( FILE *f );                        ///< Writes objects
///@}

/// 
/// \brief Reads objects from stream (FILE) f, with offset by N1 panels
/// \param f - pointer to stream - file must be opened before
/// \param Objs - vector of \ref MS2_OBJ objects
/// \param N1 - offset by N1 panels - object panels in range <n1+N1, n2+N1>
/// \param fVer - [.inp] data format version
///
	static int ReadObjs( FILE *f, std::vector<MS2_OBJ> &Objs, int N1=0, float fVer=VERSION );
/// 
/// \brief Reads objects from stream (ifstream object) f, with offset by N1 panels
/// \param f - stream object reference - file must be opened before
/// \param Objs - vector of \ref MS2_OBJ objects
/// \param N1 - offset by N1 panels - object panels in range <n1+N1, n2+N1>
/// \param fVer - [.inp] data format version
///
	static int ReadObjs( std::ifstream &f, std::vector<MS2_OBJ> &Objs, int N1=0, float fVer=VERSION );
/// 
/// \brief Writes objects to stream (FILE) f
/// \param f - pointer to stream - file must be opened before
/// \param Objs - vector of \ref MS2_OBJ objects
///
	static void WriteObjs( FILE *f, std::vector<MS2_OBJ> &Objs );
	
///@{ \name Initialization of arrays
	void InitRob( void );                          ///< initialization of scratch arrays
	void InitCommon( void );                       ///< initialization of commonly used arrays
	void InitForPanukl( void );                    ///< initialization of arrays used by PANUKL (main program)
	void InitForPress( void );                     ///< initialization of arrays used by Press program
///@}

///@{ \name Deleting of arrays
	void DeleteINP( void );
	void DeleteDAT( void );
	void DeleteDATw( void );
///@}

///@{ \name Date and time of read file
	char File_DAT_date[100];                       ///< date and time of mesh with wake file
	char File_INP_date[100];                       ///< date and time of mesh file
	char File_TXT_date[100];                       ///< date and time of results file
///@}

///@{ \name Results - objects and functions
	double* pdWyniki[22];                          ///< results array - 22 variables x number of panels
	int iNwyn;                                     ///< current number of rows of \ref pdWyniki array
	int iMwyn;                                     ///< current number of columns of \ref pdWyniki array (<=22)
	
	char TXTfile[256];                             ///< pathname of [.txt] file

	int ReadWyn( char *FileName );                 ///< reads results array from pathname FileName
	void SupWyn( void );                           ///< computes data for sides centers
	void DeleteWyn( void );                        ///< erases results objects
	int WriteWyn( char *FileName );                ///< writes results array to pathname FileName
///@}
};

#endif /*_PANUKL_STRUCT_H_*/

